/**
 * File: CheckPoint.java
 * Description: Handles all functions for checkpoints
 * 
 * Smart Houses for the Blind
 * MQP EOA 0019
 * Authors: Ben Lipson, Devin Thomas, Punit Dharani
 * 
 * Version 4/03/12
 */
package eoa.data;

public class CheckPoint 
{
	private String id, value, e1, e2, e3, e4;
	private int d1, d2, d3, d4;
	/**
	 * Default Constructor
	 */
	public CheckPoint()
	{
		id = "";
		value = "";
		
		e1 = "";
		e2 = "";
		e3 = "";
		e4 = "";
		d1 = 0;
		d2 = 0;
		d3 = 0;
		d4 = 0;
	}
	/**
	 * Constructor
	 * @param id - Tag ID
	 * @param value - Tag Value
	 */
	public CheckPoint(String id, String value, String e1, int d1, String e2, int d2,
			String e3, int d3, String e4, int d4)
	{
		this.id = id;		
		this.value = value;
		this.e1 = e1;
		this.e2 = e2;
		this.e3 = e3;
		this.e4 = e4;
		this.d1 = d1;
		this.d2 = d2;
		this.d3 = d3;
		this.d4 = d4;
	}
	/**
	 * Gets the Tag ID
	 * @return tagID
	 */
	public String getId()
	{
		return id;
	}
	/**
	 * gets tag location
	 * @return tag location
	 */
	public String getValue()
	{
		return value;
	}
	/**
	 * Get the neighbors (ID's and Distances)
	 * @return Object array containing IDs and Distances
	 */
	public Object[] getNeighbors()
	{
		return new Object[] { e1,d1,e2,d2,e3,d3,e4,d4 };
	}
	/**
	 * Return the node at the given direction
	 * i: 
	 * 	1 = North
	 * 	2 = East
	 * 	3 = South
	 * 	4 = West
	 * @param i
	 * @return ID of node at i, or null
	 */
	public Object getDir(int i)
	{
		if(i == 0)
			return e1;
		else if(i == 1)
			return e2;
		else if(i == 2)
			return e3;
		else if(i == 3)
			return e4;
		else
			return null;
	}
	/**
	 * Return the distance of the node at the given direction
	 * i: 
	 * 	1 = North
	 * 	2 = East
	 * 	3 = South
	 * 	4 = West
	 * @param i
	 * @return distance of node at i, or null
	 */
	public Object getDist(int i)
	{
		if(i == 0)
			return d1;
		else if(i == 1)
			return d2;
		else if(i == 2)
			return d3;
		else if(i == 3)
			return d4;
		else
			return null;
	}
	
	

}
