/**
 * File: DataEngine.java
 * Description: Handles all MySQL interactions
 * 
 * Smart Houses for the Blind
 * MQP EOA 0019
 * Authors: Ben Lipson, Devin Thomas, Punit Dharani
 * 
 * Version 1/23/12
 */
package eoa.data;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Map;
import java.util.HashMap;
import eoa.graph.*;
import java.util.ArrayList;
import java.util.List;


public class DataEngine 
{
	Connection con;
	int version;
	Map<String,CheckPoint> table;
	Map<String,CheckPoint> reverseLookup;
	Map<String, Vertex> graphmap;
	
	/**
	 * Default Constructor
	 */
	public DataEngine()
	{
		table = new HashMap<String, CheckPoint>();
		reverseLookup = new HashMap<String, CheckPoint>();
		version = 0;
	}
	
	public DataResult init()
	{
		
		if(connect() == 1)
		{
			System.err.println("Unable to connect to database");
			
			return DataResult.CONNECT;
		}
		//version = checkVersion();
		if(executeQuery() == 1)
		{
			System.err.println("Unable to download from the database");
			return DataResult.DOWNLOAD;
		}
		if(disconnect() == 1)
		{
			System.err.println("Unable to disconnect from database");
			return DataResult.DISCONNECT;
		}
		return DataResult.SUCCESS;
	}
	
	/**
	 * Downloads the data from the database
	 * @return 0 for success
	 * @return 1 for failure
	 */
	public int executeQuery()//QueryMode m)
	{
		try
		{
			// If the connection is closed or never initialized
			// return 1 for failure
			if(con == null || con.isClosed())
			{
				throw new SQLException("Connection is null or closed");				
			}
		}
		catch(SQLException ex)
		{
			System.err.println("SQLException at executeQuery.");
			System.err.println("Message: " + ex.getMessage());
			System.err.println("Stack Trace: " + ex.getStackTrace());
			return 1;
		}
		Statement st = null;
        ResultSet rs = null;

        try
        {
        	st = con.createStatement();
        	//if(m == QueryMode.DATA)        	
        		rs = st.executeQuery("SELECT * FROM `relate`");
        	//else if(m == QueryMode.VERSION)
        	//	rs = st.executeQuery("SELECT `version` FROM `status`;");        	
        	
        	while(rs.next())
        	{
        		//if(m == QueryMode.DATA)
        		//{
        			CheckPoint ent = new CheckPoint(rs.getString(2), rs.getString(3), 
        					rs.getString(4), rs.getInt(5), rs.getString(6), rs.getInt(7),
        					rs.getString(8), rs.getInt(9), rs.getString(10), rs.getInt(11));        			
        			table.put(rs.getString(2), ent);
        			reverseLookup.put(rs.getString(3), ent);
        		//}
        		//else if(m == QueryMode.VERSION)
        		//{
        		//	version = rs.getInt(1);
        		//}
        	}
        }
        catch(SQLException ex)
        {
			System.err.println("SQLException at executeQuery.");
			System.err.println("Message: " + ex.getMessage());
			System.err.println("Stack Trace: " + ex.getStackTrace());
        	return 1;
        }
        finally
        {
        	// cleanup. Close whatever needs to be closed.
        	try
        	{
	        	if (rs != null) {
	                rs.close();
	            }
	            if (st != null) {
	                st.close();
	            }	            
        	}catch(SQLException ex)
        	{
    			System.err.println("SQLException at executeQuery.");
    			System.err.println("Message: " + ex.getMessage());
    			System.err.println("Stack Trace: " + ex.getStackTrace()); 
        		return 1;
        	}
        	
        }
        return 0;
      
		
	}
	/**
	 * Disconnects from the database
	 * @return 0 for success
	 * @return 1 for failure
	 */
	public int disconnect()
	{
		try
		{
			if(con != null)
			{
				con.close();
				return 0;
			}
			
		}
		catch(SQLException ex)
		{
			return 1;
		}
		return 1;
	}
	/**
	 * Connects to the database
	 * @return 0 for success
	 * @return 1 for failure
	 */
	public int connect()
	{
		con = null;
        
		
        String url = "jdbc:mysql://fibi.res.wpi.net:3305/mqpeoa";
        String user = "eoa0019";
        String password = "mqp2012";

        try {
        	Class.forName("com.mysql.jdbc.Driver");
            con = DriverManager.getConnection(url, user, password);

        } catch (Exception ex) {
        	System.err.println(ex.getMessage());
        	return 1;
        }
        return 0;
	}
	/**
	 * Gets the tag's location description/name from the given tag ID
	 * @param id - tag ID
	 * @return location description
	 * @return null, if not found
	 */
	public String nameFromID(String id)
	{
		
		if(table.containsKey(id))
			return table.get(id).getValue();
		return null;
	}
	/**
	 * Gets the associated DataEntry from the given tag ID
	 * @param id - tag ID
	 * @return DataEntry associated with ID
	 * @return null, if not found
	 */
	public CheckPoint getTagData(String id)
	{
		if(table.containsKey(id))
			return table.get(id);
		return null;
	}
	/**
	 * Takes in the ID and spits out a friendly name
	 * @param name - tag ID
	 * @return friendly name
	 */
	public String IDFromName(String name)
	{
		if(reverseLookup.containsKey(name))
			return reverseLookup.get(name).getId();
		return null;
	}
	/**
	 * Updates the GraphMap
	 */
	public int updateGraph()
	{		
		// initialize the graphmap
		graphmap = new HashMap<String, Vertex>();
		
		// for each checkpoint in the tables
		for(CheckPoint c : table.values())
		{
			// add it to the graphmap
			graphmap.put(c.getId(), new Vertex(c.getId()));
		}
		
		// for each checkpoint in tables
		for(CheckPoint c : table.values())
		{
			
			// Make an array list of edges
			ArrayList<Edge> edges = new ArrayList<Edge>();
			Edge temp;
			String ename = "";
			int edist = 0;
			boolean valid = false;
			
			// for each of the checkpoint's neighbors
			for(int i = 0; i < c.getNeighbors().length; i++)
			{
				/* This is a database "hack"
				 * if i is EVEN, it's a tag ID
				 * but if i is ODD, it's going to be a distance
				 * 
				 * this is just how it's set up in the database.
				 * 
				 * if the tag ID is "NONE" set valid = false
				 * 	so the invalid edge doesn't get added
				 */
				if(i % 2 == 0)
				{
					ename = c.getNeighbors()[i].toString();
					if(ename.equals("NONE"))
						valid = false;
					else
						valid = true;
				}
				else if(i % 2 != 0 && valid)
				{
					// get the distance
					edist = Integer.parseInt(c.getNeighbors()[i].toString());
					temp = new Edge(graphmap.get(ename), edist);
					// add the edge to the list of edges
					edges.add(temp);
				}
			}
			
			// create a new vertex
			Vertex tv = graphmap.get(c.getId());
			
			// remove the old one from the graph map
			graphmap.remove(c.getId());
			
			// add the adjacencies to the vertex
			tv.adjacencies = edges.toArray(new Edge[edges.size()]);
			
			// add the vertex to the graphmap
			graphmap.put(tv.name, tv);
			
		}
		return 0;
	}
	/**
	 * Returns a vertex array of the graph
	 * @return
	 */
	public Vertex[] getGraph()
	{		
		return graphmap.values().toArray(new Vertex[graphmap.values().size()]);
	}
	
	/**
	 * Get the direction from one checkpoint to another
	 * @param src - From
	 * @param dest - To
	 * @return Compass Direction or Unknown
	 */
	public String getDirection(String src, String dest)
	{
		CheckPoint cpSrc = getTagData(src);
		
		if(cpSrc.getDir(0).equals(dest))
			return "North";
		else if(cpSrc.getDir(1).equals(dest))
			return "East";
		else if(cpSrc.getDir(2).equals(dest))
			return "South";
		else if(cpSrc.getDir(3).equals(dest))
			return "West";
		else 
			return "Unknown";
	}
	
	/**
	 * Get the distances from one tag to another 
	 * @param src - From
	 * @param dest - To
	 * @return distance from src to dest, or Unknown
	 */
	public String getDist(String src, String dest)
	{
		CheckPoint cpSrc = getTagData(src);
		if(cpSrc.getDir(0).equals(dest))
			return cpSrc.getDist(0).toString();
		else if(cpSrc.getDir(1).equals(dest))
			return cpSrc.getDist(1).toString();
		else if(cpSrc.getDir(2).equals(dest))
			return cpSrc.getDist(2).toString();
		else if(cpSrc.getDir(3).equals(dest))
			return cpSrc.getDist(3).toString();
		else 
			return "Unknown";
	}
	
	/**
	 * Returns the path (List<Vertex>) from the source Friendly-Name to dest Friendly-Name
	 * @param src - Friendly-Name Start
	 * @param dest - Friendly-Name End
	 * @return List<Vertex> path from src to dest
	 */
	public List<Vertex> getPathTo(String src, String dest)
	{
		Vertex start = null;
		Vertex end = null;
		
		// Make sure the graph is up to date
		updateGraph();
		
		// translate the friendly names into actual tag IDs
		String srcTag = IDFromName(src);
		String endTag = IDFromName(dest);
		
		// Get the graph
		Vertex[] graph = getGraph();
		
		// Traverse the graph to get the actual 
		// vertexes specified by srcTag and endTag
		
		for(Vertex v : graph)
		{			
			if(v.name == srcTag)
				start = v;
			if(v.name == endTag)
				end = v;
		}
		
		// Compute the path from start to end 		
		Graph.computePaths(start);
		
		// Get the shortest path
		List<Vertex> path = Graph.getShortestPathTo(end);
		
		// return the path
		return path;
		
	}
	
}
