/**
 * File: MqpRunner
 * Description: Runs the project
 * 
 * Smart Houses for the Blind
 * MQP EOA 0019
 * Authors: Ben Lipson, Devin Thomas, Punit Dharani
 * 
 * Version 4/03/12
 */
package eoa.main;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import eoa.data.*;
import eoa.rfid.*;
import eoa.tts.*;

public class MqpRunner {

	static RFIDListener listen;
	static RFIDScanner scanner;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) 
	{
		DataEngine engine = new DataEngine();
		VoiceEngine vengine = new VoiceEngine();
		BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
		
		// Have the engine connect to the database
		engine.connect();
		
		// Have the engine download the data
		engine.executeQuery();
		
		// Disconnect the engine from the database
		engine.disconnect();
		
		// Greet the user and instruct them how to start
		vengine.speakText("Hello. Please scan a tag to begin.");
		
		// Initialize the scanner and set it to listen
		scanner = new RFIDScanner();
		scanner.listen();
		String dest, loc;
		boolean doNav = false;
		vengine.speakText("What would you like to do? Press 1 to get nearby locations. Press 2 for navigation.");
		
		try
		{
			dest = reader.readLine();
			if(dest.charAt(0) == '1')
			{				
				CheckPoint cp = engine.getTagData(engine.IDFromName(scanner.getLast()));
				if(cp != null)
				{
					if(!cp.getDir(0).equals("NONE"))					
						vengine.speakText(engine.nameFromID(cp.getDir(0).toString()) + " is " + cp.getDist(0).toString() + " steps North");					
					if(!cp.getDir(1).equals("NONE"))					
						vengine.speakText(engine.nameFromID(cp.getDir(1).toString()) + " is " + cp.getDist(1).toString() + " steps East");
					if(!cp.getDir(2).equals("NONE"))					
						vengine.speakText(engine.nameFromID(cp.getDir(2).toString()) + " is " + cp.getDist(2).toString() + " steps South");
					if(!cp.getDir(3).equals("NONE"))					
						vengine.speakText(engine.nameFromID(cp.getDir(3).toString()) + " is " + cp.getDist(3).toString() + " steps West");
					
				}					
			}
			else if(dest.charAt(0) == '2')
				doNav = true;
		}catch(IOException ex)
		{
			
		}
		// Once the tag has been scanned, ask the user where to go.
		if(!doNav)
			return;
		vengine.speakText("Where would you like to go?");
		
		try 
		{
			// get the user's input to determine where they want to go
			dest = reader.readLine();
			
			// Get the tag ID from the given name
			loc = engine.IDFromName(dest);
			
			// If loc is not null, then the database found a match
			if(loc != null)
			{
				// let the user know where they're going to navigate to
				vengine.speakText("Navigating to " + dest);
				
				// initialize the listener and tell it the destination
				
				listen = new RFIDListener(engine, engine.getPathTo(scanner.getLast(), dest));
				
				// add the listener to the scanner
				scanner.addListener(listen);
				
				// tell it how to handle the signal received
				listen.SignalRecieved(new RFIDEvent(listen, engine.IDFromName(scanner.getLast())));
				
				// Enter an endless loop so the thread doesn't end the program prematurely 
				while(true)
				{
					
				}
			}
		} catch (IOException e) 
		{
			e.printStackTrace();
		}
	

	}

}
