/**
 * File: RFIDListener.java
 * Description: Listens for and processes events raised when the RFID reader scans a tag
 * 
 * Smart Houses for the Blind
 * MQP EOA 0019
 * Authors: Ben Lipson, Devin Thomas, Punit Dharani
 * 
 * Version 4/03/12
 */
package eoa.rfid;

import eoa.data.*;
import java.util.*;
import eoa.graph.*;
import eoa.tts.*;

public class RFIDListener 
{

	private DataEngine engine;
	private List<Vertex> _path;
	private VoiceEngine vengine;
	private Stack<Vertex> _travelled;
	private Queue<Vertex> _directions;
	
	public RFIDListener(DataEngine engine, List<Vertex> path)
	{
		_path = path;
		this.engine = engine;
		vengine = new VoiceEngine();
		_directions = new LinkedList<Vertex>();
		for(Vertex v : _path)
			_directions.add(v);
		
		_travelled = new Stack<Vertex>();
			_travelled.add(path.get(0));
		
	}
	/**
	 * Processes signals from the RFID reader
	 * @param event
	 */
	public void SignalRecieved(RFIDEvent event)
	{
		// Holds the next node in the list
		Vertex top = _directions.peek();
		
		// Holds the last traveled node
		Vertex last = _travelled.peek();
		
		// ID of the tag just scanned 
		String scannedTag = event.getCP();
				
		
		// if the current checkpoint isn't the next step in the directions list		
		if(!top.name.equals(scannedTag))
		{
			vengine.speakText("You have gone the wrong way. Turn around.");
		}
		else
		{
			// if there are still directions left
			if(_directions.size() > 1)
			{
				// remove the top of the directions queue
				_directions.remove();
				
				// The direction of the next tag from current location
				String dir = engine.getDirection(last.name, _directions.peek().name);
				
				// The distance to the next tag from the current location
				String feet = engine.getDist(last.name, _directions.peek().name);
				
				// Tell the user where they should go
				vengine.speakText("Go " + dir + " for " + feet + " feet to " + engine.nameFromID(_directions.peek().name));				
				
				// THIS DOESN'T WORK PROPERLY
				
				_travelled.add(_directions.peek());
			}
			else // otherwise
			{
				vengine.speakText("You have arrived at your destination");
			}
		}
	}
}
