/**
 * File: RFIDScanner.java
 * Description: Sets up and runs the RFID scanner.
 * 
 * Smart Houses for the Blind
 * MQP EOA 0019
 * Authors: Ben Lipson, Devin Thomas, Punit Dharani
 * 
 * Based on some code from Oracle (http://www.oracle.com/technetwork/java/index-jsp-141752.html)
 * 
 * Version 4/03/12
 */
package eoa.rfid;
import java.io.*;
import java.util.*;

import eoa.data.*;
import javax.comm.*;

import eoa.tts.VoiceEngine;

public class RFIDScanner  extends Thread implements Runnable, SerialPortEventListener
{

	private List<RFIDListener> listeners;
	static CommPortIdentifier portId;
    static Enumeration portList;
    private boolean run = true;
    InputStream inputStream;
    SerialPort serialPort;
    Thread readThread;
    int bufsize;
    DataEngine engine;
    VoiceEngine vengine;
	public RFIDScanner()
	{
		engine = new DataEngine();
		vengine = new VoiceEngine();
		listeners = new ArrayList<RFIDListener>();
	}
	public void Activate()
	{
		if(engine.init() != DataResult.SUCCESS)
		{			
			return;
		}
		portList = CommPortIdentifier.getPortIdentifiers();

        while (portList.hasMoreElements()) 
        {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) 
            {
                if (portId.getName().equals("COM4")) 
                {
                    init();
                    break;
                }
            }
        }
	}
	public void Stop()
	{
		run = false;
		
		
	}
	private volatile String last;
	public synchronized String getLast()
	{
		return last;
	}
	private void init()
	{
		try 
		{
			serialPort = (SerialPort) portId.open("SimpleReadApp", 2000);
		} 
		catch (PortInUseException e) {System.out.println(e);
		}
		try 
		{        	
			bufsize = 12;        	
		    inputStream = serialPort.getInputStream();
		    

		} catch (IOException e) 
		{
			System.out.println(e);
		}
		try 
		{
			serialPort.addEventListener(this);
		} 
		catch (TooManyListenersException e) 
		{
			System.out.println(e);
		}
		serialPort.notifyOnDataAvailable(true);		
		try 
		{        	
			serialPort.setSerialPortParams(9600,
					SerialPort.DATABITS_8,
		            SerialPort.STOPBITS_1,
		            SerialPort.PARITY_NONE);
		} 
		catch (UnsupportedCommOperationException e) 
		{
			System.out.println(e);
		}
		while(run)
		{
			//do nothing
		}
		
		
	}
    private String dest = "";
    public String listen()
    {
    	engine = new DataEngine();
		vengine = new VoiceEngine();
    	if(engine.init() != DataResult.SUCCESS)
		{			
			return null;
		}
		portList = CommPortIdentifier.getPortIdentifiers();

        while (portList.hasMoreElements()) 
        {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) 
            {
                if (portId.getName().equals("COM4")) 
                {
                    init();
                    break;
                }
            }
        }
        return dest;
        
    }
    public synchronized void addListener(RFIDListener l )
    {
    	listeners.add(l);
    }
    public synchronized void removeListener(RFIDListener l )
    {
    	listeners.remove(l);
    }
    public void serialEvent(SerialPortEvent event) {
    	
        switch(event.getEventType()) {
        case SerialPortEvent.BI:
        case SerialPortEvent.OE:
        case SerialPortEvent.FE:
        case SerialPortEvent.PE:
        case SerialPortEvent.CD:
        case SerialPortEvent.CTS:
        case SerialPortEvent.DSR:
        case SerialPortEvent.RI:        	
        case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
            break;
        case SerialPortEvent.DATA_AVAILABLE:
        	StringBuffer readBuffer = new StringBuffer();
            int c;
            try {
                 while ((c=inputStream.read()) != 10){
                	 readBuffer.append((char) c);
                 }
           
                 inputStream.close();
                 String output = readBuffer.toString().trim();
                 
                 String loc = engine.nameFromID(output);
                 if(loc == null)
                	 loc = "{INVALID LOCATION}";
                 vengine.speakText(String.format("You are at location %s.", loc));
                 for(RFIDListener l : listeners)
                 {                	 
                	 l.SignalRecieved(new RFIDEvent(this, output));
                 }
                 dest = loc;
                 last = loc;
                 this.Stop();
                 
                                 
            } catch (IOException e) {}
            
        }
    }
}
